"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.addAfterIxsNeededIfRepayTokenIsRENEC = exports.addPreIxsNeededIfRepayTokenIsRENEC = exports.buildLiquidateIxs = void 0;
const liquidateObligationAndRedeemReserveCollateral_1 = require("../instructions/liquidateObligationAndRedeemReserveCollateral.js");
const spl_token_1 = require("@solana/spl-token");
const spl_token0_2_0_1 = require("@solana/spl-token0.2.0");
const web3_js_1 = require("@solana/web3.js");
const __1 = require("../index.js");
const constants_1 = require("../core/constants.js");
const PRICE_PAIRS = (0, constants_1.getPricePair)();
const REUSD_PRICE = (0, constants_1.getReUSDPricePair)();
const REVND_PRICE = constants_1.PRICE_PAIRS_MAINNET_TEST_REVND_RENGN[0];
const RENGN_PRICE = constants_1.PRICE_PAIRS_MAINNET_TEST_REVND_RENGN[1];
const buildLiquidateIxs = async (connection, obligation, liquidityAmount, depositReserve, borrowReserve, publicKey) => {
    const ixs = [];
    const lendingMarket = (0, constants_1.getRelendInfo)()[0];
    const programId = (0, __1.getProgramId)();
    const obligationAccountInfo = await connection.getAccountInfo(obligation);
    let obligationDetails = null;
    const depositReserves = [];
    const borrowReserves = [];
    if (obligationAccountInfo) {
        obligationDetails = (0, __1.parseObligation)(web3_js_1.PublicKey.default, obligationAccountInfo).info;
        obligationDetails.deposits.forEach((deposit) => {
            depositReserves.push(deposit.depositReserve);
        });
        obligationDetails.borrows.forEach((borrow) => {
            borrowReserves.push(borrow.borrowReserve);
        });
    }
    const selectedDepositReserve = lendingMarket.reserves.find((r) => r.address === depositReserve.toBase58());
    const selectedBorrowReserve = lendingMarket.reserves.find((r) => r.address === borrowReserve.toBase58());
    if (!selectedDepositReserve || !selectedBorrowReserve) {
        throw new Error("Couldn't find collateral or loan reserve.");
    }
    // refresh reserve
    const allReserveAddresses = [
        ...Array.from([...depositReserves.map((e) => e.toBase58()), ...borrowReserves.map((e) => e.toBase58())]),
    ];
    const uniqueAllReserveAddresses = [...new Set(allReserveAddresses)];
    const pda = (0, constants_1.getPDA)();
    uniqueAllReserveAddresses.forEach((reserveAddress) => {
        const reserveInfo = lendingMarket.reserves.find((reserve) => reserve.address === reserveAddress);
        if (!reserveInfo) {
            throw new Error(`Could not find asset ${reserveAddress} in reserves`);
        }
        const symbol = reserveInfo.liquidityToken.symbol;
        let pair;
        if (symbol === constants_1.RE_USD) {
            pair = REUSD_PRICE;
        }
        else if (symbol === constants_1.RE_VND) {
            pair = REVND_PRICE;
        }
        else if (symbol === constants_1.RE_NGN) {
            pair = RENGN_PRICE;
        }
        else {
            pair = PRICE_PAIRS.find((pair) => pair.baseSymbol === reserveInfo.liquidityToken.symbol);
        }
        const oracleProduct = pda.product(new web3_js_1.PublicKey(pair.quoteToken), new web3_js_1.PublicKey(pair.baseToken));
        const refreshReserveIx = (0, __1.refreshReserveInstruction)(new web3_js_1.PublicKey(reserveAddress), programId, oracleProduct.key, new web3_js_1.PublicKey(reserveInfo.pythOracle), new web3_js_1.PublicKey(reserveInfo.switchboardOracle));
        ixs.push(refreshReserveIx);
    });
    const refreshObligationIx = (0, __1.refreshObligationInstruction)(obligation, depositReserves, borrowReserves, programId);
    ixs.push(refreshObligationIx);
    // get account that will be repaying the reserve liquidity
    const repayAccount = await spl_token_1.Token.getAssociatedTokenAddress(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, new web3_js_1.PublicKey(selectedBorrowReserve.liquidityToken.mint), // MINT RENEC
    publicKey);
    const userRepayAccountInfo = await connection.getAccountInfo(repayAccount);
    if (selectedBorrowReserve.liquidityToken.symbol === constants_1.RENEC) {
        await (0, exports.addPreIxsNeededIfRepayTokenIsRENEC)(connection, liquidityAmount, publicKey, ixs, repayAccount, selectedBorrowReserve);
    }
    else if (!userRepayAccountInfo) {
        throw new Error("Insufficient funds.");
    }
    // Withdraw Collateral Account
    const rewardedWithdrawalCollateralAccount = await spl_token_1.Token.getAssociatedTokenAddress(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, new web3_js_1.PublicKey(selectedDepositReserve.collateralMintAddress), publicKey);
    const rewardedWithdrawalCollateralAccountInfo = await connection.getAccountInfo(rewardedWithdrawalCollateralAccount);
    if (!rewardedWithdrawalCollateralAccountInfo) {
        const createUserCollateralAccountIx = (0, spl_token0_2_0_1.createAssociatedTokenAccountInstruction)(publicKey, rewardedWithdrawalCollateralAccount, publicKey, new web3_js_1.PublicKey(selectedDepositReserve.collateralMintAddress));
        ixs.push(createUserCollateralAccountIx);
    }
    // Withdraw Liquidity Account
    const rewardedWithdrawalLiquidityAccount = await (0, spl_token0_2_0_1.getAssociatedTokenAddress)(new web3_js_1.PublicKey(selectedDepositReserve.liquidityToken.mint), publicKey);
    const rewardedWithdrawalLiquidityAccountInfo = await connection.getAccountInfo(rewardedWithdrawalLiquidityAccount);
    if (!rewardedWithdrawalLiquidityAccountInfo &&
        !(selectedBorrowReserve.liquidityToken.symbol === constants_1.RENEC && selectedDepositReserve.liquidityToken.symbol === constants_1.RENEC)) {
        const createUserLiquidityAccountIx = (0, spl_token0_2_0_1.createAssociatedTokenAccountInstruction)(publicKey, rewardedWithdrawalLiquidityAccount, publicKey, new web3_js_1.PublicKey(selectedDepositReserve.liquidityToken.mint));
        ixs.push(createUserLiquidityAccountIx);
    }
    ixs.push((0, liquidateObligationAndRedeemReserveCollateral_1.LiquidateObligationAndRedeemReserveCollateral)(liquidityAmount, repayAccount, rewardedWithdrawalCollateralAccount, rewardedWithdrawalLiquidityAccount, new web3_js_1.PublicKey(selectedBorrowReserve.address), new web3_js_1.PublicKey(selectedBorrowReserve.liquidityAddress), new web3_js_1.PublicKey(selectedDepositReserve.address), new web3_js_1.PublicKey(selectedDepositReserve.collateralMintAddress), new web3_js_1.PublicKey(selectedDepositReserve.collateralSupplyAddress), new web3_js_1.PublicKey(selectedDepositReserve.liquidityAddress), new web3_js_1.PublicKey(selectedDepositReserve.liquidityFeeReceiverAddress), obligation, new web3_js_1.PublicKey(lendingMarket.address), new web3_js_1.PublicKey(lendingMarket.authorityAddress), publicKey, programId));
    await (0, exports.addAfterIxsNeededIfRepayTokenIsRENEC)(publicKey, ixs, selectedDepositReserve, selectedBorrowReserve);
    return ixs;
};
exports.buildLiquidateIxs = buildLiquidateIxs;
const addPreIxsNeededIfRepayTokenIsRENEC = async (connection, amount, publicKey, instructions, userTokenAccountAddress, selectedBorrowReserve) => {
    if (selectedBorrowReserve.liquidityToken.symbol === constants_1.RENEC) {
        const userWRENECAccountInfo = await connection.getAccountInfo(userTokenAccountAddress);
        const rentExempt = await spl_token_1.Token.getMinBalanceRentForExemptAccount(connection);
        const transferLamportsIx = web3_js_1.SystemProgram.transfer({
            fromPubkey: publicKey,
            toPubkey: userTokenAccountAddress,
            lamports: (userWRENECAccountInfo ? 0 : rentExempt) + parseInt(amount.toString(), 10),
        });
        const syncIx = (0, __1.syncNative)(userTokenAccountAddress);
        if (userWRENECAccountInfo) {
            instructions.push(transferLamportsIx);
        }
        else {
            const createUserWRENECAccountIx = (0, spl_token0_2_0_1.createAssociatedTokenAccountInstruction)(publicKey, userTokenAccountAddress, publicKey, spl_token_1.NATIVE_MINT);
            instructions.push(createUserWRENECAccountIx);
            instructions.push(transferLamportsIx);
        }
        instructions.push(syncIx);
    }
};
exports.addPreIxsNeededIfRepayTokenIsRENEC = addPreIxsNeededIfRepayTokenIsRENEC;
const addAfterIxsNeededIfRepayTokenIsRENEC = async (publicKey, instructions, selectedDepositReserve, selectedBorrowReserve) => {
    if (selectedBorrowReserve.liquidityToken.symbol === constants_1.RENEC || selectedDepositReserve.liquidityToken.symbol === constants_1.RENEC) {
        const WRENECAccount = await (0, spl_token0_2_0_1.getAssociatedTokenAddress)(spl_token_1.NATIVE_MINT, publicKey);
        const closeWRENECAccountIx = spl_token_1.Token.createCloseAccountInstruction(spl_token_1.TOKEN_PROGRAM_ID, WRENECAccount, publicKey, publicKey, []);
        instructions.push(closeWRENECAccountIx);
    }
};
exports.addAfterIxsNeededIfRepayTokenIsRENEC = addAfterIxsNeededIfRepayTokenIsRENEC;
